
/* %W% last mod on %G% at %U% */
/* Full SCCS path name: (%P%) */

/*******************************************************************

	NAME: clock.c
	
	AUTHOR: K. Schroeder
	
	PURPOSE: Allow program to do nothing for a specified time interval.
	
	CONTENTS:
		WaitMilSec() - do nothing for given milliseconds.
		GetPresentTime()  - determine user, system and elapse time
				for the program
		TimeToString(stime) - returns a character string for a 
				formated string of the time.
	
*******************************************************************/

#include "local.h"
#include <stdio.h>
#ifdef DOS
#include <time.h>
#else
#include <time.h>
#include <sys/times.h>
#include <sys/time.h>
#include <unistd.h>
#endif			/* DOS */
#include <sys/types.h>
#include "son.h"

/* definitions	*/

/* external	*/

/* structures and global/semiglobal variable */

/* Prototypes */

LOCAL_FN void
GetPresentTime(
#if NeedFunctionPrototypes
	long *user,
	long *system,
	long *elapse
#endif
);

LOCAL_FN void MinuteSecondAppendString();

/*******************************************************************
	NAME: WaitMilSec
	
	DESCRIPTION: Waits for atleast msec milliseconds.
	
	RETURNS: none
*******************************************************************/


void
WaitMilSec(msec)
int msec;
{
	long c1, c2;
	c1 = clock();
	while (1) {
		c2 = clock();	/* clock returns time in micro-seconds */

		/*
		** clock could wrap around.  Lets just set the initial 
		** time back to 0.  This occurs infrequently and we will
		** just return a little earlier.
		*/
		if (c2 < c1)
			c1 = 0;

		if ((c2 - c1) * 1000 > msec * CLOCKS_PER_SEC) {
			return;
		}
	}
}

/*******************************************************************
	NAME: GetTime
	
	DESCRIPTION: 
		Get the time user, system and elapse time for a program.
	
	USAGE:
		sTime *stime;		
		stime = GetTime( (sTime *)NULL);
		# execute program
		stime = GetTime(stime);
		# stime->user, stime->system, stime->elapse has the 
		# user, system and elapse time (long value);

		# May call multiple times. Time is relative to the inital call
		stime = GetTime( stime);  
		sFree((char *)stime);


	RETURNS: pointer to the sTime structure.
*******************************************************************/

sTime *
GetTime(stime)
sTime *stime;
{
	int long user, system, elapse;

	if (stime == NULL) {
		stime = (sTime *) new(sizeof(sTime) );
		GetPresentTime(
		    &stime->orig_user,
		    &stime->orig_system,
		    &stime->orig_elapse);
		return (stime);
	}

	GetPresentTime(&user, &system, &elapse);

	stime->user = user - stime->orig_user;
	stime->system = system - stime->orig_system;
	stime->elapse = elapse - stime->orig_elapse;

	return (stime);
}

LOCAL_FN void
GetPresentTime(user, system, elapse)
long *user;
long *system;
long *elapse;
{
#ifdef DOS
	long c1;
	c1 = clock() / CLOCKS_PER_SEC;
	*elapse = c1;
	*user = c1;
	*system = (long) 0;
#else		/* Not DOS */
        struct tms buffer;
        long clk_tck;
        time_t time2,time1;

	times(&buffer);
	clk_tck = sysconf(_SC_CLK_TCK);
	if ( clk_tck == -1 ) {
		clk_tck = (long)1;
	}
	*elapse = time( (time_t)0 );
	*user   = buffer.tms_utime/clk_tck;
	*system = buffer.tms_stime/clk_tck;
#endif		/* DOS*/
}

char *
TraceTime(){
	struct tm *tm;
	time_t  timeofday;
	static char tim[12];

	timeofday = time((time_t *) 0);
	tm = localtime(&timeofday);
	sprintf(tim, "%02d%02d %02d:%02d",
	    tm->tm_mon + 1,
	    tm->tm_mday,
	    tm->tm_hour,
	    tm->tm_min);

	return (tim);
}

/*******************************************************************
	NAME: TimeToString
	
	DESCRIPTION: 
		Returns a string containing the user, system and real time
		in the following examples format.

		"1:16 user, 1:00 system, 0:07 real"
	
		The string is stores statically and should NOT be deallocated.

	EXAMPLE USAGE:
		sTime *stime;		
		printf("Analysis time per frequency (mm:ss) -- %s\n",
			TimeToString(stime));

	RETURNS: pointer to statically declared character pointer.

*******************************************************************/
char *
TimeToString(stime)
sTime *stime;
{
	static char str[80];

	strcpy(str, "");
	MinuteSecondAppendString(str, stime->user, "user, ");
	MinuteSecondAppendString(str, stime->system, "system, ");
	MinuteSecondAppendString(str, stime->elapse, "real");
	return(str);
}

LOCAL_FN void
MinuteSecondAppendString(str, time, append_str)
char *str;
long time;
char *append_str;
{
	long minutes, seconds;
	char timstr[30];
	char *fmt;

	/*Calculate and print the user time*/
	minutes = time / (long) 60;
	seconds = time - minutes * (long) 60;
	if (seconds > 9) {
		fmt = "%d:%d %s";
	}
	else {
		fmt = "%d:0%d %s";
	}
	sprintf(timstr, fmt, (int) minutes, (int) seconds, append_str);
	strcat(str, timstr);
}

/*******************************************************************
	NAME: WaitMilSec
	
	DESCRIPTION: Waits for atleast msec milliseconds.
	
	RETURNS: none
*******************************************************************/
static char *month[12] = {
        "January", "February", "March", "April", "May", "June",
        "July", "August", "September", "October", "November", "December"
};


char *
GetDateString()
{
	static char str[30];
	time_t  timeofday;
	struct tm *tm;

	timeofday = time( (time_t *)0 );
	tm = localtime( &timeofday );
        sprintf(str,"%s %d, %d",
		month[tm->tm_mon], 
		tm->tm_mday,
		tm->tm_year+1900);
	return(str);
}
